@@ -8,16 +8,16 @@
 #ifndef PAINT_EXAMPLE_H_
 #define PAINT_EXAMPLE_H_
 
-#include <Wt/WApplication>
+#include <Wt/WContainerWidget>
 
 using namespace Wt;
 
 class ShapesWidget;
 
-class PaintExample : public WApplication
+class PaintExample : public WContainerWidget
 {
 public:
-  PaintExample(const WEnvironment& env);
+  PaintExample(WContainerWidget *root, bool showTitle=true);
 
 private:
   ShapesWidget *shapes_;
